﻿###############################################################################
#                                                                             #
#                   Deleting Distribution Group for Office 365                #
#                             For Primary Group                               #
#                                  V3.9.8.0                                   #
#                       (C)2006-2020 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_errors_count
{
Param($Group)
$S="ErrorsCount[$($Group)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function Banner
{
Param($MenuLines)
$MarginWidth=2
$MarginChar=" "
$MaxWidth=0;
$BorderTextColor="Magenta"
ForEach ($Line In $MenuLines)
        {
        If ($Line.Length -gt $MaxWidth) {$MaxWidth=$Line.Length}
        }
If ($MaxWidth -gt 0)
   {
   Write-Host "╔" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╗" -ForegroundColor $BorderTextColor
   ForEach ($Line In $MenuLines)
           {
           Write-Host "║" -NoNewline -ForegroundColor $BorderTextColor
           Write-Host ("{0}{1}{2}" -f $MarginChar.PadLeft($MarginWidth),$Line.PadRight($MaxWidth),$MarginChar.PadLeft($MarginWidth)) -NoNewline -ForegroundColor White
           Write-Host "║" -ForegroundColor $BorderTextColor
           }
   Write-Host "╚" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╝" -ForegroundColor $BorderTextColor
   }
}
###############################################################################
function welcome
{
$BannerText="Deleting distribution group for Office 365 $($Version)",
            "(C) 2006-2020 KoXo Developpement"
Banner $BannerText
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword,$ProxyEnabled,$ProxyUser,$ProxyPassword)
$error.Clear()
#Proxy
if ($ProxyEnabled)
   {
   if (![String]::IsNullorEmpty($ProxyUser))
      { 
      $SecuredPassword=ConvertTo-SecureString $ProxyPassword -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $ProxyUser, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection with Internet Explorer proxy parameters, User=$($ProxyUser)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection with Internet Explorer proxy parameters"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
if ($ProxyEnabled)
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -SessionOption $PSSessionOption -ErrorAction SilentlyContinue
   }
   else
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
   }
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
 return $Cred
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
    $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
    log_message "ERROR" $S
	}
}
###############################################################################
function distribution_group_exists
{
Param([string] $GroupId)
$error.Clear()
$Group=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -arg $GroupId {Param($Id) Get-DistributionGroup -Identity $Id}
return (![string]::IsNullOrEmpty($Group))
}
###############################################################################
function delete_distribution_group
{
Param($GroupName,
      $GroupDisplayName,
	    $GroupAlias,
      $GroupEmail)
$error.Clear()
%{Invoke-Command -Session $Script:RS {Param ($Id) Remove-DistributionGroup -Identity $Id -Confirm:$false}-arg $GroupEmail}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
  $S="Successfully delete distribution group, Name=""$($GroupName)"", DisplayName=""$($GroupDisplayName)"", Email=""$($GroupEmail)"", Alias=""$($GroupAlias)"""
	log_message "OK" $S
	}
	else
	{
	$global:errors_count++
  $S="Deleting distribution group, Name=""$($GroupName)"", DisplayName=""$($GroupDisplayName)"", Email=""$($GroupEmail)"", Alias=""$($GroupAlias)"". Error = $($error[0])"
  log_message "ERROR" $S
  }
}
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V3.9.8.0"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%OFFICE365_PASSWORD%}"))
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%PROXY_PASSWORD%}"))
#
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
connect $Office365Url $Office365LoginName $Office365LoginPassword $ProxyEnabled $ProxyUser $ProxyPassword
if ($global:total_errors_count -eq 0)
   {
   ### BEGIN_REPETITION ###
   $global:errors_count=0
   # KoXo Administrator's variables
   $DistributionGroupName="%SECONDARY_GROUP%"
   $DistributionGroupDisplayName="%SECONDARY_GROUP_DESC%"
   $DistributionGroupAlias="#LOWER{#REMOVE_SPACE{%SECONDARY_GROUP%}}"
   $DistributionGroupEmail="%SECONDARY_GROUP_EMAIL%"
   #
   log_message "INFO" "Distribution group for secondary group : Name=""$($DistributionGroupName)"", DisplayName=""$($DistributionGroupDisplayName)"", Email=""$($DistributionGroupEmail)"", Alias=""$($DistributionGroupAlias)"""
   if (distribution_group_exists($DistributionGroupEmail))
      {
      $S="Distribution group, Name=""$($DistributionGroupName)"", DisplayName=""$($DistributionGroupDisplayName)"", Email=""$($DistributionGroupEmail)"", Alias=""$($DistributionGroupAlias)"""
      write-host $S
      delete_distribution_group $DistributionGroupName $DistributionGroupDisplayName $DistributionGroupAlias $DistributionGroupEmail
      }
      else
      {
      $global:errors_count++
      $S="Distribution group, Name=""$($DistributionGroupName)"", DisplayName=""$($DistributionGroupDisplayName)"", Email=""$($DistributionGroupEmail)"", Alias=""$($DistributionGroupAlias)"" does not exists"
      write-host $S -foregroundcolor "red"
      log_message "ERROR" $S
      }
   log_errors_count $DistributionGroupName
   $global:total_errors_count=$global:total_errors_count+$global:errors_count
   ### END_REPETITION ###
   close
   }
log_total_errors_count
