﻿###############################################################################
#                                                                             #
#                      Delete a team for Microsoft Teams                      #
#                                    V3.9.8.0                                 #
#                       (C)2006-2020 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_errors_count
{
Param($ID)
$S="ErrorsCount[$($ID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_warnings_count
{
$S="TotalWarningsCount=$($global:total_warnings_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_warnings_count
{
Param($ID)
$S="WarningsCount[$($ID)]=$($global:warnings_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "5")
   {
   $S="Powershell's version must be greater or equal to 5.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function Banner
{
Param($MenuLines)
$MarginWidth=2
$MarginChar=" "
$MaxWidth=0;
$BorderTextColor="Magenta"
ForEach ($Line In $MenuLines)
        {
        If ($Line.Length -gt $MaxWidth) {$MaxWidth=$Line.Length}
        }
If ($MaxWidth -gt 0)
   {
   Write-Host "╔" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╗" -ForegroundColor $BorderTextColor
   ForEach ($Line In $MenuLines)
           {
           Write-Host "║" -NoNewline -ForegroundColor $BorderTextColor
           Write-Host ("{0}{1}{2}" -f $MarginChar.PadLeft($MarginWidth),$Line.PadRight($MaxWidth),$MarginChar.PadLeft($MarginWidth)) -NoNewline -ForegroundColor White
           Write-Host "║" -ForegroundColor $BorderTextColor
           }
   Write-Host "╚" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╝" -ForegroundColor $BorderTextColor
   }
}
###############################################################################
function welcome
{
$BannerText="Deleting a team for Microsoft Teams $($Version)",
            "(C) 2006-2020 KoXo Developpement"
Banner $BannerText
}
###############################################################################
function get_cred
{
Param([string] $MyName,$MyPassword)
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName)) )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
return $Cred
}
###############################################################################
function display_user
{
Param ($UserDisplayName,$UserEmail)
Write-Host "User: """ -nonewline
Write-Host $UserDisplayName -foregroundcolor "Green" -nonewline
Write-Host """ Email: """ -nonewline
Write-Host $UserEmail -foregroundcolor "Green" -nonewline
Write-host """"	
}
###############################################################################
function set_proxy
{
Param($Enabled,$Url,$Port,$User,$Password)
if ($Enabled)
   {
   $Cred=get_cred $User $Password
   $Proxy=-Join("http://",$Url,":",$Port)
   Invoke-WebRequest -Proxy $Proxy -ProxyCredential $Cred -ErrorAction SilentlyContinue
   }
}
###############################################################################
function teams_module_is_present
{
 if (Get-Module -ListAvailable -Name MicrosoftTeams) 
    {
    Return $True
    }
    else
    {
    Return $False
    }
}
###############################################################################
function connect_teams
{
Param($Cred)
$error.Clear()
Connect-MicrosoftTeams -Credential $Cred -ErrorAction SilentlyContinue
Return ([String]::IsNullorEmpty($error[0]))
}
###############################################################################
function team_add_users
{
Param($Team,$Users,$Role)
If ((![String]::IsNullorEmpty($Users)) -and ($Team -ne $null) -and (![String]::IsNullorEmpty($Role)))
   {
   $Options=[System.StringSplitOptions]::RemoveEmptyEntries
   $ListOfUsers=$Users.Split(",",$Options)
   If ($ListOfUsers.Count -gt 0)
      {
      $ListOfUsers=$ListOfUsers | Select-Object -Unique
      }
   ForEach ($User In $ListOfUsers)
           {
           $error.Clear()
           Add-TeamUser -GroupId $Team.GroupId -User $User -Role $Role -ErrorAction SilentlyContinue
           If ([String]::IsNullorEmpty($error[0]))
	          {
              $S="Successfully add user ""$($User)"" as $($Role.ToLower()) to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
              Write-host $S -foregroundcolor "Green"
	          log_message "OK" $S
	          }
	          else
	          {
	          $global:errors_count++
              $S="Adding user ""$($User)"" as $($Role.ToLower()) to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
              Write-host $S -foregroundcolor "Red"
              log_message "ERROR" $S
	          }
           }
   }
}
###############################################################################
function team_remove_users
{
Param($Team,$Users)
If ((![String]::IsNullorEmpty($Users)) -and ($Team -ne $null))
   {
   $Options=[System.StringSplitOptions]::RemoveEmptyEntries
   $ListOfUsers=$Users.Split(",",$Options)
   If ($ListOfUsers.Count -gt 0) 
      {
      $ListOfUsers=$ListOfUsers | Select-Object -Unique
      }
   ForEach ($User In $ListOfUsers)
           {
           $error.Clear()
           Remove-TeamUser -GroupId $Team.GroupId -User $User -ErrorAction SilentlyContinue
           If ([String]::IsNullorEmpty($error[0]))
	          {
              $S="Successfully remove user ""$($User)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
              Write-host $S -foregroundcolor "Green"
	          log_message "OK" $S
	          }
	          else
	          {
	          $global:errors_count++
              $S="Removing user ""$($User)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
              Write-host $S -foregroundcolor "Red"
              log_message "ERROR" $S
	          }
           }
   }
}
###############################################################################
function team_exists
{
Param($MailNickName)
Return ((Get-Team -MailNickName $MailNickName) -ne $null)
}
###############################################################################
function team_remove
{
Param($MailNickName)
$Team=Get-Team -MailNickName $MailNickName -ErrorAction SilentlyContinue
If ($Team -ne $null)
   {
   $error.Clear()
   Remove-Team -GroupId $Team.GroupId -ErrorAction SilentlyContinue
   If ([String]::IsNullorEmpty($error[0]))
	          {
              $S="Successfully delete team ""$($MailNickName)"""
              Write-host $S -foregroundcolor "Green"
	          log_message "OK" $S
	          }
	          else
	          {
	          $global:errors_count++
              $S="Creating team ""$($MailNickName)"""
              Write-host $S -foregroundcolor "Red"
              log_message "ERROR" $S
	          }
    }
    else
    {
    $global:errors_count++
    $S="Cannot find team ""$($MailNickName)"""
    Write-host $S -foregroundcolor "Red"
    log_message "ERROR" $S
    }
}
###############################################################################
function team_channel_exists
{
Param($GroupId,$DisplayName)
Return ((Get-TeamChannel -GroupId $GroupId | Where DisplayName -EQ $DisplayName) -ne $null)
}
###############################################################################
# Channels : channel1,channel2,channel3 ...
# Channel : Displayname | Displayname%Description | Displayname%Description%MembershipType | Displayname%Description%MembershipType%Owner
# Warning : MembershipType and Owner are nor supported on all versions of modules
function team_add_channels
{
Param($Team,$Channels)
If (![String]::IsNullorEmpty($Channels)) 
   {
   $Options=[System.StringSplitOptions]::RemoveEmptyEntries
   $ListOfChannels=$Channels.Split(",",$Options)
   ForEach ($Channel In $ListOfChannels)
           {
           $Params=@{"GroupId"=$Team.GroupID;"ErrorAction"="SilentlyContinue"}
           $ChannelSplit=$Channel.Split("%",$Options)
           $DisplayName=$ChannelSplit[0].Trim()
           $Description=$DisplayName
           # Description in 2nd field
           If ($ChannelSplit.Count -gt 1)
                            {
                            $Description=$ChannelSplit[1]
                            }
           $Params+=@{"DisplayName"=$DisplayName;"Description"=$Description}
           # "Private" without owner in fields
           If ($ChannelSplit.Count -ge 3) 
              {
              If ($ChannelSplit[2].ToLower().Trim() -eq "private")
                 {
                 $Params+=@{"MembershipType"="Private"}
                 }
              }
           # "Private" with specified owner
           If ($ChannelSplit.Count -eq 4) 
              {
              If ($ChannelSplit[2].ToLower().Trim() -eq "private")
                 { 
                 $Params+=@{"Owner"=$ChannelSplit[3]}
                 }
               }
           $error.Clear()
           $RetChannel=New-TeamChannel @Params
           If ([String]::IsNullorEmpty($error[0]))
              {
              $S="Successfully add channel ""$($DisplayName)"" (Id=$($RetChannel.Id)) to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
              Write-host $S -foregroundcolor "Green"
	          log_message "OK" $S
              }
	          else
	          {
	          $global:errors_count++
              $S="Adding channel team ""$($DisplayName)"" to team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
              Write-host $S -foregroundcolor "Red"
              log_message "ERROR" $S
              }
           }
   }
}
###############################################################################
function team_remove_channel
{
Param($Team,$DisplayName)
If (![String]::IsNullorEmpty($DisplayName)) 
   {
   $error.Clear()
   $Params=@{"GroupId"=$Team.GroupId;"DisplayName"=$DisplayName;"ErrorAction"="SilentlyContinue"}
   Remove-TeamChannel @Params
   If ([String]::IsNullorEmpty($error[0]))
      {
      $S="Successfully remove channel ""$($DisplayName)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
      Write-host $S -foregroundcolor "Green"
      log_message "OK" $S
      }
      else
      {
      $global:errors_count++
      $S="Removing channel team ""$($DisplayName)"" from team ""$($Team.DisplayName)"" (MailNickName=$($Team.MailNickName))"
      Write-host $S -foregroundcolor "Red"
      log_message "ERROR" $S
      }
   }
}
###############################################################################
# Warning : Template is not supported on all versions of modules
# Warning : Template : mut be "EDU_Class" or "PLC_Class"
function team_create
{
Param($DisplayName,$Description,$MailNickName,$Visibility,$Owners,$DefaultOwner,$Template,$Channels)
If (team_exists($MailNickName))
   {
   $S="Teams ""$($MailNickName)"" already exists !"
   Write-Host $S -Foregroundcolor "Red"
   Log_Message "ERROR" $S
   $global:total_errors_count++
   Return $null
   Break  
   }
If (![String]::IsNullorEmpty($Owners))
   {
   $Options=[System.StringSplitOptions]::RemoveEmptyEntries
   $ListOfOwners=$Owners.Split(",",$Options)
   If ($ListOfOwners.Count -gt 0)
      {
      $Owner=$ListOfOwners[0]
      }
   }
   else
   {
   $Owner=$DefaultOwner
   }
If ([String]::IsNullorEmpty($Description))
   {
   $Description=$Name
   }
$Params=@{"DisplayName"=$DisplayName;"MailNickName"=$MailNickName;"Owner"=$Owner;"ErrorAction"="SilentlyContinue"}
If (![String]::IsNullorEmpty($Description))
   {
   $Params+=@{"Description"=$Description}
   }
If (![String]::IsNullorEmpty($Template))
   {
   $Params+=@{"Template"=$Template}
   }
   else
   {
   $Params+=@{"Visibility"=$Visibility}
   }
$error.Clear()
$Team=New-Team @Params
If ([String]::IsNullorEmpty($error[0]))
	          {
              $S="Successfully create team ""$($DisplayName)"" (MailNickName=$($MailNickName), Id=$($Team.GroupId))"
              Write-host $S -foregroundcolor "Green"
	          log_message "OK" $S
              team_add_channels $Team $Channels
	          }
	          else
	          {
	          $global:errors_count++
              $S="Creating team ""$($DisplayName)"" (MailNickName=$($MailNickName))"
              Write-host $S -foregroundcolor "Red"
              log_message "ERROR" $S
	          }
 team_add_users $Team $Members "Member"
 team_add_users $Team $Owners "Owner"
 Return $Team
}
###############################################################################
# Modify : DisplayName, Description, Visibility, Owners, Members, Channels
function team_modify
{
Param($DisplayName,$Description,$MailNickName,$Visibility,$Owners,$DefaultOwner,$Members,$Channels)
If (![String]::IsNullorEmpty($Description))
   {
   $Description=$DisplayName;
   }
$Team=Get-Team -MailNickName $MailNickName -ErrorAction SilentlyContinue
If ($Team -ne $null)
   {
   # Modify team
   $Params=@{"GroupId"=$Team.GroupId;"DisplayName"=$DisplayName;"Description"=$Description;"ErrorAction"="SilentlyContinue"}
   If ([String]::IsNullorEmpty($Template))
      { 
      If ($Team.Visibility -ne "HiddenMembership")
         {
         $Params+=@{"Visibility"=$Visibility}
         }
      }
   $error.Clear()
   $Ret=Set-Team @Params
   If ([String]::IsNullorEmpty($error[0]))
      {
      $S="Modifying team ""$($DisplayName)"" (GroupID=$($Team.GroupId), MailNickName=$($MailNickName))"
      Write-host $S -foregroundcolor "Green"
      log_message "OK" $S
      }
      else
      {
      $global:errors_count++
      $S="Modifying team ""$($DisplayName)"" (MailNickName=$($MailNickName))"
      Write-host $S -foregroundcolor "Red"
      log_message "ERROR" $S
      }
   # Synchronize Channels
   $ListOfChannels=$Channels.Split(",",[System.StringSplitOptions]::RemoveEmptyEntries)
   $Params=@{"GroupId"=$Team.GroupId;"ErrorAction"="SilentlyContinue"}
   $error.Clear()
   $TeamChannels=Get-TeamChannel @Params
   ForEach ($TempChannel in $ListOfChannels)
           {
           # Add channel if necessary
           If ($TeamChannels.DisplayName.IndexOf($TempChannel.Split('%')[0]) -eq -1)
              {
              team_add_channels $Team $TempChannel 
              }
           }
   $ListOfChannels2=New-Object System.Collections.Generic.List[string]
   ForEach ($TempChannel in $ListOfChannels)
           {
           $ListOfChannels2.Add($TempChannel.Split('%')[0])
           }
   ForEach ($TeamChannel in $TeamChannels)
           {
           # Remove a channel if necessary, and not "General"
           $ChannelDisplayName=$TeamChannel.DisplayName
           If (($ListOfChannels2.IndexOf($ChannelDisplayName) -eq -1) -and ($ChannelDisplayName.ToLower() -ne "general"))
              {
              team_remove_channel $Team $ChannelDisplayName
              }
           }
   # Synchronize members & owners
   $TeamsUsers=Get-TeamUser -GroupId $Team.GroupId -ErrorAction SilentlyContinue
   If ($TeamsUsers -ne $null)
      {
      # Existing members and owners
      $ExistingOwners=New-Object System.Collections.Generic.List[string]
      $ExistingMembers=New-Object System.Collections.Generic.List[string]
      ForEach ($TeamUser in $TeamsUsers)
              {
              If ($TeamUser.Role -eq "Member")
                 {
                 $ExistingMembers.Add($TeamUser.User.ToLower())
                 }
                 else
                 {
                 $ExistingOwners.Add($TeamUser.User.ToLower())
                 }
              }
      $TotalOwners=$ExistingOwners.Count
      # Members and owners
      $Owners=$Owners.ToLower()
      $ListOfOwners=$Owners.Split(",",[System.StringSplitOptions]::RemoveEmptyEntries)
      If ($ListOfOwners.Count -gt 0) 
         {
         $ListOfOwners=$ListOfOwners | Select-Object -Unique
         }
      $Members=$Members.ToLower()
      $ListOfMembers=$Members.Split(",",[System.StringSplitOptions]::RemoveEmptyEntries)
      If ($ListOfMembers.Count -gt 0) 
         {
         $ListOfMembers=$ListOfMembers | Select-Object -Unique
         }
      # Add members and owners
      ForEach ($TempUser In $ListOfMembers)
              {
              If ($ExistingMembers.IndexOf($TempUser) -eq -1)
                 {
                 team_add_users $Team $TempUser "Member"
                 }
              }
      ForEach ($TempUser In $ListOfOwners)
              {
              If ($ExistingOwners.IndexOf($TempUser) -eq -1)
                 {
                 team_add_users $Team $TempUser "Owner"
                 $TotalOwners++
                 }
              }
      # Remove members and owners
      ForEach ($Tempuser In $ExistingMembers)
              {
              If ($ListOfMembers.IndexOf($TempUser) -eq -1)
                 {
                 team_remove_users $Team $TempUser
                 }
              }
      ForEach ($Tempuser In $ExistingOwners)
              {
              If ($ListOfOwners.IndexOf($TempUser) -eq -1)
                 {
                 If ($TotalOwners -eq 1)
                    {
                    if ($DefaultOwner -ne $TempUser)
                       {
                       team_add_users $Team $DefaultOwner "Owner"
                       team_remove_users $Team $TempUser
                       $TotalOwners--
                       }
                    }
                    else
                    {
                    If ($TotalOwners -gt 1)
                       {
                       team_remove_users $Team $TempUser
                       $TotalOwners--
                       }
                    }
                 }
              }
      }
      else
      {
      $global:errors_count++
      $S="Cannot read users from team ""$($MailNickName)"""
      Write-host $S -foregroundcolor "Red"
      log_message "ERROR" $S
      }
   }
   else
   {
   $global:errors_count++
   $S="Cannot find team : MailNickName=""$($MailNickName)"""
   Write-host $S -foregroundcolor "Red"
   log_message "ERROR" $S
   }
}
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V3.9.8.0"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%OFFICE365_PASSWORD%}"))
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%PROXY_PASSWORD%}"))
$ProxyUrl="%PROXY_ADDRESS%"
$ProxyPort=%PROXY_PORT%
#
# Init
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
$global:total_warnings_count=0
#
welcome
check_version
If (teams_module_is_present)
   {
   Import-Module -Name MicrosoftTeams
   Write-host "PowerShell module : " -foregroundcolor "Yellow" -NoNewline
   Write-host ("Microsoft Teams V",(Get-Module -Name MicrosoftTeams).Version) -foregroundcolor "Green" -Separator ""
   set_proxy $ProxyEnabled $ProxyUrl $ProxyPort $ProxyUser $ProxyPassword
   $Cred=get_cred $Office365LoginName $Office365LoginPassword
   If (connect_teams $Cred)
      {
      ### BEGIN_REPETITION ###
      $global:errors_count=0
      $Secondary_Group="%SECONDARY_GROUP%"
      $SecondaryGroup="%SECONDARYGROUP%"
      $MailNickName=-Join("teams-",$SecondaryGroup.ToLower())
      team_remove $MailNickName
      log_errors_count $Secondary_Group
      $global:total_errors_count=$global:total_errors_count+$global:errors_count
      ### END_REPETITION ###	
      }
   } 
   else
   {
   $S="Module MicrosoftTeams is not present !"
   Write-Host $S -Foregroundcolor "Red"
   Log_Message "ERROR" $S
   $global:total_errors_count++
   }
log_total_errors_count
log_total_warnings_count