﻿###############################################################################
#                                                                             #
#                        Creating Mailbox for Office 365                      #
#                                  V3.9.8.0                                   #
#                       (C)2006-2020 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_errors_count
{
Param($ID)
$S="ErrorsCount[$($ID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function Banner
{
Param($MenuLines)
$MarginWidth=2
$MarginChar=" "
$MaxWidth=0;
$BorderTextColor="Magenta"
ForEach ($Line In $MenuLines)
        {
        If ($Line.Length -gt $MaxWidth) {$MaxWidth=$Line.Length}
        }
If ($MaxWidth -gt 0)
   {
   Write-Host "╔" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╗" -ForegroundColor $BorderTextColor
   ForEach ($Line In $MenuLines)
           {
           Write-Host "║" -NoNewline -ForegroundColor $BorderTextColor
           Write-Host ("{0}{1}{2}" -f $MarginChar.PadLeft($MarginWidth),$Line.PadRight($MaxWidth),$MarginChar.PadLeft($MarginWidth)) -NoNewline -ForegroundColor White
           Write-Host "║" -ForegroundColor $BorderTextColor
           }
   Write-Host "╚" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╝" -ForegroundColor $BorderTextColor
   }
}
###############################################################################
function welcome
{
$BannerText="Creating user for Office 365 $($Version)",
            "(C) 2006-2020 KoXo Developpement"
Banner $BannerText
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword,$ProxyEnabled,$ProxyUser,$ProxyPassword)
$error.Clear()
#Proxy
if ($ProxyEnabled)
   {
   if (![String]::IsNullorEmpty($ProxyUser))
      { 
      $SecuredPassword=ConvertTo-SecureString $ProxyPassword -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $ProxyUser, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection with Internet Explorer proxy parameters, User=$($ProxyUser)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection with Internet Explorer proxy parameters"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
if ($ProxyEnabled)
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -SessionOption $PSSessionOption -ErrorAction SilentlyContinue
   }
   else
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
   }
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
 return $Cred
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
    $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
    log_message "ERROR" $S
	}
}
###############################################################################
function user_exists
{
Param([string] $email)
$error.Clear()
$User=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -ArgumentList $email {Param($id) Get-User -Identity $id}
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function create_user
{
Param($UserEmail,
      $UserPassword,
      $UserMustChangePassword,
      $UserHiddenFromAddressLists,
      $UserDisplayName,
      $UserTitle,$UserFirstName,$UserLastName,
      $UserInitials,
      $UserPhone,$UserMobilePhone,$UserFaxNumber,
      $UserWebPage,
      $UserDepartment,$UserOffice,$UserCompany,
      $UserEmail1,$UserEmail2,$UserEmail3,$UserEmail4,$UserEmail5,
      $UserNotes,
      $UserPicture,
      $MemberOf,
      $CustomAttribute1,$CustomAttribute2,$CustomAttribute3,$CustomAttribute4,
      $CustomAttribute5,$CustomAttribute6,$CustomAttribute7,$CustomAttribute8,
      $CustomAttribute9,$CustomAttribute10,$CustomAttribute11,$CustomAttribute12,
      $CustomAttribute13,$CustomAttribute14,$CustomAttribute15)
$UserIdentity=$UserEmail.Split("@")[0]
if ($UserMustChangePassword -isnot[Boolean]) {$UserMustChangePassword=$false}
if ($UserHiddenFromAddressLists -isnot[Boolean]) {$UserHiddenFromAddressLists=$false}
$UserNotes=$UserNotes -replace ("/","`n")
$error.Clear()
# Secure the password
if (![String]::IsNullorEmpty($UserPassword))
			{
				$SecuredPassword = ConvertTo-SecureString $UserPassword -AsPlainText -Force
			}
			else
			{
				$SecuredPassword = $UserPassword
			}
%{Invoke-Command -Session $Script:RS {param ($name,$pwd,$email,$resetpwd,$firstName,$lastName,$displayName,$initials) new-mailbox -Name $name -Password $pwd -MicrosoftOnlineServicesID $email -ResetPasswordOnNextLogon $resetpwd -FirstName $firstName -LastName $lastName -DisplayName $displayName -Initials $initials}-arg $UserIdentity,$SecuredPassword,$UserEmail,$UserMustChangePassword,$UserFirstName,$UserLastName,$UserDisplayName,$UserInitials}>$Results    	
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully create mailbox for $($UserDisplayName) ($($UserEmail))"
	log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Creating mailbox for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])"
    log_message "ERROR" $S
    }
# Build the list of Proxy Email Addresses
$EmailAddresses=New-Object System.Collections.ArrayList
if (![String]::IsNullorEmpty($UserEmail))
   {
	$index=$EmailAddresses.Add("SIP:$($UserEmail)")
  $index=$EmailAddresses.Add("SMTP:$($UserEmail)")
	if (![String]::IsNullorEmpty($UserEmail1))
	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail1)")
	   }
	if (![String]::IsNullorEmpty($UserEmail2))
	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail2)")
	   }
	if (![String]::IsNullorEmpty($UserEmail3))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail3)")
   	   }
	if (![String]::IsNullorEmpty($UserEmail4))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail4)")
   	   }
	if (![String]::IsNullorEmpty($UserEmail5))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail5)")
   	   }
	}
# Add Proxy Email Addresses and Custom Attributes    
$error.Clear()
%{Invoke-Command -Session $Script:RS {param ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16,$17,$18) set-mailbox -Identity $1 -EmailAddresses $2 -CustomAttribute1 $3 -CustomAttribute2 $4 -CustomAttribute3 $5 -CustomAttribute4 $6 -CustomAttribute5 $7 -CustomAttribute6 $8 -CustomAttribute7 $9 -CustomAttribute8 $10 -CustomAttribute9 $11 -CustomAttribute10 $12 -CustomAttribute11 $13 -CustomAttribute12 $14 -CustomAttribute13 $15 -CustomAttribute14 $16 -CustomAttribute15 $17 -HiddenFromAddressListsEnabled $18}-arg $UserIdentity,$EmailAddresses,$CustomAttribute1,$CustomAttribute2,$CustomAttribute3,$CustomAttribute4,$CustomAttribute5,$CustomAttribute6,$CustomAttribute7,$CustomAttribute8,$CustomAttribute9,$CustomAttribute10,$CustomAttribute11,$CustomAttribute12,$CustomAttribute13,$CustomAttribute14,$CustomAttribute15,$UserHiddenFromAddressLists}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
	$S="Successfully update mailbox for $($UserDisplayName) ($($UserEmail))"
    log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Updating mailbox for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])"
    log_message "ERROR" $S
    }    
# Add User Misc. Attributes    
$error.Clear()
%{Invoke-Command -Session $Script:RS {param ($id,$title,$phone,$mobile,$fax,$webpage,$department,$office,$company,$notes) set-user -Identity $id -Title $title -Phone $phone -MobilePhone $mobile -Fax $fax -WebPage $webpage -Department $department -Office $office -Company $company -Notes $notes}-arg $UserIdentity,$UserTitle,$UserPhone,$UserMobilePhone,$UserFaxNumber,$UserWebPage,$UserDepartment,$UserOffice,$UserCompany,$UserNotes}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
	$S="Successfully update user's informations for $($UserDisplayName) ($($UserEmail))"
    log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Updating user's informatios for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])"
    log_message "ERROR" $S
    }
# Add membership to distribution groups
$Groups=$MemberOf.Split(",")
ForEach ($Group in $Groups)
        {
        if (![String]::IsNullorEmpty($Group))
           {
           $error.Clear()
           %{Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue {param ($grp,$memb) add-distributiongroupmember -Identity $grp -Member $memb}-arg $Group,$UserIdentity}
           if ([String]::IsNullorEmpty($error[0]))
   	          {
              $S="Successfully add distribution group ""$($Group)"" for $($UserDisplayName) ($($UserEmail))"
              write-host $S
              log_message "OK" $S
              }
           }
        }  
# Add user's picture or photo / must be smaller than 10KB
if (![String]::IsNullorEmpty($UserPicture))
   {
   If ((Test-Path $UserPicture) -eq $True)
      {
      $error.Clear()
      %{Invoke-Command -Session $Script:RS {param ($id,$picture) set-userphoto -Identity $id -PictureData ([System.IO.File]::ReadAllBytes($picture)) -Confirm:$false}-arg $UserEmail,$UserPicture}>$Results
      if ([String]::IsNullorEmpty($error[0]))
	       {
	       $FileSize=(Get-Item $UserPicture).Length / 1024
	       $S="Successfully add user's picture ""$($UserPicture)"" ({0:n1}KB) for $($UserDisplayName) ($($UserEmail))" -f $FileSize
         log_message "OK" $S
	       }
	       else
	       {
	       $global:errors_count++
         $FileSize=(Get-Item $UserPicture).Length / 1024
         $S="Adding user's picture ""$($UserPicture)"" ({0:n1}KB) for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])" -f $FileSize
         log_message "ERROR" $S
         }
      }   
   }
}
###############################################################################
function display_user
{
Param ($UserDisplayName,$UserEmail)
Write-Host "User: """ -nonewline
Write-Host $UserDisplayName -foregroundcolor "Green" -nonewline
Write-Host """ Email: """ -nonewline
Write-Host $UserEmail -foregroundcolor "Green" -nonewline
Write-host """"	
}
###############################################################################
function exec
{
Param($UserEmail,
      $UserPassword,$UserMustChangePassword,$UserHiddenFromAddressLists,
      $UserDisplayName,$UserTitle,$UserFirstName,$UserLastName,$UserInitials,
      $UserPhone,$UserMobilePhone,$UserFaxNumber,$UserWebPage,$UserDepartment,$UserOffice,$UserCompany,
      $UserEmail1,$UserEmail2,$UserEmail3,$UserEmail4,$UserEmail5,
      $UserNotes,
      $UserPicture,
      $MemberOf,
      $CustomAttribute1,$CustomAttribute2,$CustomAttribute3,
      $CustomAttribute4,$CustomAttribute5,$CustomAttribute6,
      $CustomAttribute7,$CustomAttribute8,$CustomAttribute9,
      $CustomAttribute10,$CustomAttribute11,$CustomAttribute12,
      $CustomAttribute13,$CustomAttribute14,$CustomAttribute15
      )
$UserDisplayName=$UserDisplayName.Trim()
log_message "Office 365" "$($UserDisplayName) ($($UserEmail))"
if (user_exists($UserEmail))
   {
   $global:errors_count++
   $S="User:""$($UserEmail)"" already exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
   else
   {
   display_user	$UserDisplayName $UserEmail
   create_user $UserEmail $UserPassword $UserMustChangePassword $UserHiddenFromAddressLists $UserDisplayName $UserTitle $UserFirstName $UserLastName $UserInitials $UserPhone $UserMobilePhone $UserFaxNumber $UserWebPage $UserDepartment $UserOffice $UserCompany $UserEmail1 $UserEmail2 $UserEmail3 $UserEmail4 $UserEmail5 $UserNotes $UserPicture $MemberOf $CustomAttribute1 $CustomAttribute2  $CustomAttribute3 $CustomAttribute4 $CustomAttribute5 $CustomAttribute6 $CustomAttribute7 $CustomAttribute8 $CustomAttribute9 $CustomAttribute10 $CustomAttribute11 $CustomAttribute12 $CustomAttribute13 $CustomAttribute14 $CustomAttribute15
   }
log_errors_count($UserEmail)   
$global:total_errors_count=$global:total_errors_count+$global:errors_count
}
#
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V3.9.8.0"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%OFFICE365_PASSWORD%}"))
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%PROXY_PASSWORD%}"))
#
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
connect $Office365Url $Office365LoginName $Office365LoginPassword $ProxyEnabled $ProxyUser $ProxyPassword
if ($global:total_errors_count -eq 0)
   {
   ### BEGIN_REPETITION ###
   $global:errors_count=0
   $UserMustChangePassword=$#BOOL{%OFFICE365_RESET_PASSWORD_ON_NEXT_LOGON%}
   $UserHiddenFromAddressLists=$#BOOL{%OFFICE365_USER_HIDDEN_FROM_ADDRESS_LISTS%}
   $UserEmail="%USER_EMAIL%"
   $UserPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("#BASE64{%USER_PASSWORD%}"))
   $UserDisplayName="%USER_DISPLAY_NAME%"
   $UserTitle="%USER_TITLE%"
   $UserFirstName="%USER_FIRST_NAME%"
   $UserLastName="%USER_LAST_NAME%"
   $UserInitials="%USER_INITIALS%"
   $UserPhone="%USER_PHONE%"
   $UserMobilePhone="%USER_MOBILE_PHONE%"
   $UserFaxNumber="%USER_FAX_NUMBER%"
   $UserWebPage="%USER_WEB_PAGE%"
   $UserDepartment="%PRIMARY_GROUP% / %SECONDARY_GROUP%"
   $UserOffice="%SECONDARY_GROUP%"
   $UserCompany="%COMPANY%"
   $UserEmail1="%USER_EMAILS[1]%"
   $UserEmail2="%USER_EMAILS[2]%"
   $UserEmail3="%USER_EMAILS[3]%"
   $UserEmail4="%USER_EMAILS[4]%"
   $UserEmail5="%USER_EMAILS[5]%"
   $UserNotes="%USER_REMARKS%"
   $UserPicture="%USER_ID_THUMBNAIL_PICTURE%"
   $CustomAttribute1 ="#LOWER{#REMOVE_SPACE{%PRIMARY_GROUP%}}"
   $CustomAttribute2 ="#LOWER{#REMOVE_SPACE{%SECONDARY_GROUP%}}"
   $CustomAttribute3 =""
   $CustomAttribute4 =""
   $CustomAttribute5 =""
   $CustomAttribute6 =""
   $CustomAttribute7 =""
   $CustomAttribute8 =""
   $CustomAttribute9 =""
   $CustomAttribute10=""
   $CustomAttribute11=""
   $CustomAttribute12=""
   $CustomAttribute13=""
   $CustomAttribute14=""
   $CustomAttribute15=""
   $MemberOf="$($CustomAttribute1),$($CustomAttribute2)"
   exec $UserEmail $UserPassword $UserMustChangePassword $UserHiddenFromAddressLists $UserDisplayName $UserTitle $UserFirstName $UserLastName $UserInitials $UserPhone $UserMobilePhone $UserFaxNumber $UserWebPage $UserDepartment $UserOffice $UserCompany $UserEmail1 $UserEmail2 $UserEmail3 $UserEmail4 $UserEmail5 $UserNotes $UserPicture $MemberOf $CustomAttribute1 $CustomAttribute2  $CustomAttribute3 $CustomAttribute4 $CustomAttribute5 $CustomAttribute6 $CustomAttribute7 $CustomAttribute8 $CustomAttribute9 $CustomAttribute10 $CustomAttribute11 $CustomAttribute12 $CustomAttribute13 $CustomAttribute14 $CustomAttribute15
   ### END_REPETITION ###
   close
   }
log_total_errors_count
